@@ -1,6 +1,6 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006, 2007
    Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
@@ -86,10 +86,10 @@ struct input_block
 	u_s;	/* INPUT_STRING */
       struct
 	{
-	  FILE *fp;		/* input file handle */
-	  bool end : 1;		/* true if peek has seen EOF */
-	  bool close : 1;	/* true if we should close file on pop */
-	  bool advance_line : 1; /* track previous start_of_input_line */
+	  FILE *fp;		     /* input file handle */
+	  bool_bitfield end : 1;     /* true if peek has seen EOF */
+	  bool_bitfield close : 1;   /* true if we should close file on pop */
+	  bool_bitfield advance : 1; /* track previous start_of_input_line */
 	}
 	u_f;	/* INPUT_FILE */
       builtin_func *func;	/* pointer to macro's function */
@@ -197,7 +197,7 @@ push_file (FILE *fp, const char *title, bool close)
   i->u.u_f.fp = fp;
   i->u.u_f.end = false;
   i->u.u_f.close = close;
-  i->u.u_f.advance_line = start_of_input_line;
+  i->u.u_f.advance = start_of_input_line;
   output_current_line = -1;
 
   i->prev = isp;
@@ -352,7 +352,7 @@ pop_input (void)
 	  M4ERROR ((warning_status, errno, "error reading file"));
 	  retcode = EXIT_FAILURE;
 	}
-      start_of_input_line = isp->u.u_f.advance_line;
+      start_of_input_line = isp->u.u_f.advance;
       output_current_line = -1;
       break;
 
@@ -752,15 +752,6 @@ set_comment (const char *bc, const char *ec)
 
 #ifdef ENABLE_CHANGEWORD
 
-static void
-init_pattern_buffer (struct re_pattern_buffer *buf)
-{
-  buf->translate = NULL;
-  buf->fastmap = NULL;
-  buf->buffer = NULL;
-  buf->allocated = 0;
-}
-
 void
 set_word_regexp (const char *regexp)
 {
@@ -776,7 +767,7 @@ set_word_regexp (const char *regexp)
     }
 
   /* Dry run to see whether the new expression is compilable.  */
-  init_pattern_buffer (&new_word_regexp);
+  init_pattern_buffer (&new_word_regexp, NULL);
   msg = re_compile_pattern (regexp, strlen (regexp), &new_word_regexp);
   regfree (&new_word_regexp);
 
@@ -817,22 +808,23 @@ set_word_regexp (const char *regexp)
 #endif /* ENABLE_CHANGEWORD */
 
 
-/*-------------------------------------------------------------------------.
-| Parse and return a single token from the input stream.  A token can	   |
-| either be TOKEN_EOF, if the input_stack is empty; it can be TOKEN_STRING |
-| for a quoted string; TOKEN_WORD for something that is a potential macro  |
-| name; and TOKEN_SIMPLE for any single character that is not a part of	   |
-| any of the previous types.						   |
-|									   |
-| Next_token () return the token type, and passes back a pointer to the	   |
-| token data through TD.  The token text is collected on the obstack	   |
-| token_stack, which never contains more than one token text at a time.	   |
-| The storage pointed to by the fields in TD is therefore subject to	   |
-| change the next time next_token () is called.				   |
-`-------------------------------------------------------------------------*/
+/*--------------------------------------------------------------------.
+| Parse and return a single token from the input stream.  A token     |
+| can either be TOKEN_EOF, if the input_stack is empty; it can be     |
+| TOKEN_STRING for a quoted string; TOKEN_WORD for something that is  |
+| a potential macro name; and TOKEN_SIMPLE for any single character   |
+| that is not a part of any of the previous types.  If LINE is not    |
+| NULL, set *LINE to the line where the token starts.                 |
+|                                                                     |
+| Next_token () return the token type, and passes back a pointer to   |
+| the token data through TD.  The token text is collected on the      |
+| obstack token_stack, which never contains more than one token text  |
+| at a time.  The storage pointed to by the fields in TD is           |
+| therefore subject to change the next time next_token () is called.  |
+`--------------------------------------------------------------------*/
 
 token_type
-next_token (token_data *td)
+next_token (token_data *td, int *line)
 {
   int ch;
   int quote_level;
@@ -842,9 +834,11 @@ next_token (token_data *td)
   char *orig_text = NULL;
 #endif
   const char *file;
-  int line;
+  int dummy;
 
   obstack_free (&token_stack, token_bottom);
+  if (!line)
+    line = &dummy;
 
  /* Can't consume character until after CHAR_MACRO is handled.  */
   ch = peek_input ();
@@ -869,7 +863,7 @@ next_token (token_data *td)
 
   next_char (); /* Consume character we already peeked at.  */
   file = current_file;
-  line = current_line;
+  *line = current_line;
   if (MATCH (ch, bcomm.string, true))
     {
       obstack_grow (&token_stack, bcomm.string, bcomm.length);
@@ -881,7 +875,7 @@ next_token (token_data *td)
       else
 	/* current_file changed to "" if we see CHAR_EOF, use the
 	   previous value we stored earlier.  */
-	M4ERROR_AT_LINE ((EXIT_FAILURE, 0, file, line,
+	M4ERROR_AT_LINE ((EXIT_FAILURE, 0, file, *line,
 			  "ERROR: end of file in comment"));
 
       type = TOKEN_STRING;
@@ -964,7 +958,7 @@ next_token (token_data *td)
 	  if (ch == CHAR_EOF)
 	    /* current_file changed to "" if we see CHAR_EOF, use
 	       the previous value we stored earlier.  */
-	    M4ERROR_AT_LINE ((EXIT_FAILURE, 0, file, line,
+	    M4ERROR_AT_LINE ((EXIT_FAILURE, 0, file, *line,
 			      "ERROR: end of file in string"));
 
 	  if (MATCH (ch, rquote.string, true))
