@@ -1,6 +1,7 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 90, 91, 92, 93, 94 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2006, 2007 Free
+   Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,8 +24,8 @@
 
 #include "m4.h"
 
-static void expand_macro _((symbol *));
-static void expand_token _((struct obstack *, token_type, token_data *));
+static void expand_macro (symbol *);
+static void expand_token (struct obstack *, token_type, token_data *, int);
 
 /* Current recursion level in expand_macro ().  */
 int expansion_level = 0;
@@ -32,6 +33,23 @@ int expansion_level = 0;
 /* The number of the current call of expand_macro ().  */
 static int macro_call_id = 0;
 
+/* The shared stack of collected arguments for macro calls; as each
+   argument is collected, it is finished and its location stored in
+   argv_stack.  Normally, this stack can be used simultaneously by
+   multiple macro calls; the exception is when an outer macro has
+   generated some text, then calls a nested macro, in which case the
+   nested macro must use a local stack to leave the unfinished text
+   alone.  Too bad obstack.h does not provide an easy way to reopen a
+   finished object for further growth, but in practice this does not
+   hurt us too much.  */
+static struct obstack argc_stack;
+
+/* The shared stack of pointers to collected arguments for macro
+   calls.  This object is never finished; we exploit the fact that
+   obstack_blank is documented to take a negative size to reduce the
+   size again.  */
+static struct obstack argv_stack;
+
 /*----------------------------------------------------------------------.
 | This function read all input, and expands each token, one at a time.  |
 `----------------------------------------------------------------------*/
@@ -41,9 +59,16 @@ expand_input (void)
 {
   token_type t;
   token_data td;
+  int line;
 
-  while ((t = next_token (&td)) != TOKEN_EOF)
-    expand_token ((struct obstack *) NULL, t, &td);
+  obstack_init (&argc_stack);
+  obstack_init (&argv_stack);
+
+  while ((t = next_token (&td, &line)) != TOKEN_EOF)
+    expand_token ((struct obstack *) NULL, t, &td, line);
+
+  obstack_free (&argc_stack, NULL);
+  obstack_free (&argv_stack, NULL);
 }
 
 
@@ -55,7 +80,7 @@ expand_input (void)
 `------------------------------------------------------------------------*/
 
 static void
-expand_token (struct obstack *obs, token_type t, token_data *td)
+expand_token (struct obstack *obs, token_type t, token_data *td, int line)
 {
   symbol *sym;
 
@@ -65,9 +90,13 @@ expand_token (struct obstack *obs, token_type t, token_data *td)
     case TOKEN_MACDEF:
       break;
 
+    case TOKEN_OPEN:
+    case TOKEN_COMMA:
+    case TOKEN_CLOSE:
     case TOKEN_SIMPLE:
     case TOKEN_STRING:
-      shipout_text (obs, TOKEN_DATA_TEXT (td), strlen (TOKEN_DATA_TEXT (td)));
+      shipout_text (obs, TOKEN_DATA_TEXT (td), strlen (TOKEN_DATA_TEXT (td)),
+		    line);
       break;
 
     case TOKEN_WORD:
@@ -75,14 +104,14 @@ expand_token (struct obstack *obs, token_type t, token_data *td)
       if (sym == NULL || SYMBOL_TYPE (sym) == TOKEN_VOID
 	  || (SYMBOL_TYPE (sym) == TOKEN_FUNC
 	      && SYMBOL_BLIND_NO_ARGS (sym)
-	      && peek_input () != '('))
+	      && peek_token () != TOKEN_OPEN))
 	{
 #ifdef ENABLE_CHANGEWORD
 	  shipout_text (obs, TOKEN_DATA_ORIG_TEXT (td),
-			strlen (TOKEN_DATA_ORIG_TEXT (td)));
+			strlen (TOKEN_DATA_ORIG_TEXT (td)), line);
 #else
 	  shipout_text (obs, TOKEN_DATA_TEXT (td),
-			strlen (TOKEN_DATA_TEXT (td)));
+			strlen (TOKEN_DATA_TEXT (td)), line);
 #endif
 	}
       else
@@ -91,7 +120,7 @@ expand_token (struct obstack *obs, token_type t, token_data *td)
 
     default:
       M4ERROR ((warning_status, 0,
-		"INTERNAL ERROR: Bad token type in expand_token ()"));
+		"INTERNAL ERROR: bad token type in expand_token ()"));
       abort ();
     }
 }
@@ -107,22 +136,24 @@ expand_token (struct obstack *obs, token_type t, token_data *td)
 | obstack OBS, indirectly through expand_token ().			   |
 `-------------------------------------------------------------------------*/
 
-static boolean
+static bool
 expand_argument (struct obstack *obs, token_data *argp)
 {
   token_type t;
   token_data td;
   char *text;
   int paren_level;
+  const char *file = current_file;
+  int line = current_line;
 
   TOKEN_DATA_TYPE (argp) = TOKEN_VOID;
 
   /* Skip leading white space.  */
   do
     {
-      t = next_token (&td);
+      t = next_token (&td, NULL);
     }
-  while (t == TOKEN_SIMPLE && isspace (*TOKEN_DATA_TEXT (&td)));
+  while (t == TOKEN_SIMPLE && isspace (to_uchar (*TOKEN_DATA_TEXT (&td))));
 
   paren_level = 0;
 
@@ -131,38 +162,43 @@ expand_argument (struct obstack *obs, token_data *argp)
 
       switch (t)
 	{			/* TOKSW */
-	case TOKEN_SIMPLE:
-	  text = TOKEN_DATA_TEXT (&td);
-	  if ((*text == ',' || *text == ')') && paren_level == 0)
+	case TOKEN_COMMA:
+	case TOKEN_CLOSE:
+	  if (paren_level == 0)
 	    {
-
 	      /* The argument MUST be finished, whether we want it or not.  */
 	      obstack_1grow (obs, '\0');
-	      text = obstack_finish (obs);
+	      text = (char *) obstack_finish (obs);
 
 	      if (TOKEN_DATA_TYPE (argp) == TOKEN_VOID)
 		{
 		  TOKEN_DATA_TYPE (argp) = TOKEN_TEXT;
 		  TOKEN_DATA_TEXT (argp) = text;
 		}
-	      return (boolean) (*TOKEN_DATA_TEXT (&td) == ',');
+	      return t == TOKEN_COMMA;
 	    }
+	  /* fallthru */
+	case TOKEN_OPEN:
+	case TOKEN_SIMPLE:
+	  text = TOKEN_DATA_TEXT (&td);
 
 	  if (*text == '(')
 	    paren_level++;
 	  else if (*text == ')')
 	    paren_level--;
-	  expand_token (obs, t, &td);
+	  expand_token (obs, t, &td, line);
 	  break;
 
 	case TOKEN_EOF:
-	  M4ERROR ((EXIT_FAILURE, 0,
-		    "ERROR: EOF in argument list"));
+	  /* current_file changed to "" if we see TOKEN_EOF, use the
+	     previous value we stored earlier.  */
+	  M4ERROR_AT_LINE ((EXIT_FAILURE, 0, file, line,
+			    "ERROR: end of file in argument list"));
 	  break;
 
 	case TOKEN_WORD:
 	case TOKEN_STRING:
-	  expand_token (obs, t, &td);
+	  expand_token (obs, t, &td, line);
 	  break;
 
 	case TOKEN_MACDEF:
@@ -170,17 +206,16 @@ expand_argument (struct obstack *obs, token_data *argp)
 	    {
 	      TOKEN_DATA_TYPE (argp) = TOKEN_FUNC;
 	      TOKEN_DATA_FUNC (argp) = TOKEN_DATA_FUNC (&td);
-	      TOKEN_DATA_FUNC_TRACED (argp) = TOKEN_DATA_FUNC_TRACED (&td);
 	    }
 	  break;
 
 	default:
 	  M4ERROR ((warning_status, 0,
-		    "INTERNAL ERROR: Bad token type in expand_argument ()"));
+		    "INTERNAL ERROR: bad token type in expand_argument ()"));
 	  abort ();
 	}
 
-      t = next_token (&td);
+      t = next_token (&td, NULL);
     }
 }
 
@@ -194,21 +229,19 @@ static void
 collect_arguments (symbol *sym, struct obstack *argptr,
 		   struct obstack *arguments)
 {
-  int ch;			/* lookahead for ( */
   token_data td;
   token_data *tdp;
-  boolean more_args;
-  boolean groks_macro_args = SYMBOL_MACRO_ARGS (sym);
+  bool more_args;
+  bool groks_macro_args = SYMBOL_MACRO_ARGS (sym);
 
   TOKEN_DATA_TYPE (&td) = TOKEN_TEXT;
   TOKEN_DATA_TEXT (&td) = SYMBOL_NAME (sym);
-  tdp = (token_data *) obstack_copy (arguments, (voidstar) &td, sizeof (td));
-  obstack_grow (argptr, (voidstar) &tdp, sizeof (tdp));
+  tdp = (token_data *) obstack_copy (arguments, &td, sizeof td);
+  obstack_ptr_grow (argptr, tdp);
 
-  ch = peek_input ();
-  if (ch == '(')
+  if (peek_token () == TOKEN_OPEN)
     {
-      next_token (&td);		/* gobble parenthesis */
+      next_token (&td, NULL); /* gobble parenthesis */
       do
 	{
 	  more_args = expand_argument (arguments, &td);
@@ -216,11 +249,10 @@ collect_arguments (symbol *sym, struct obstack *argptr,
 	  if (!groks_macro_args && TOKEN_DATA_TYPE (&td) == TOKEN_FUNC)
 	    {
 	      TOKEN_DATA_TYPE (&td) = TOKEN_TEXT;
-	      TOKEN_DATA_TEXT (&td) = "";
+	      TOKEN_DATA_TEXT (&td) = (char *) "";
 	    }
-	  tdp = (token_data *)
-	    obstack_copy (arguments, (voidstar) &td, sizeof (td));
-	  obstack_grow (argptr, (voidstar) &tdp, sizeof (tdp));
+	  tdp = (token_data *) obstack_copy (arguments, &td, sizeof td);
+	  obstack_ptr_grow (argptr, tdp);
 	}
       while (more_args);
     }
@@ -252,7 +284,7 @@ call_macro (symbol *sym, int argc, token_data **argv,
 
     default:
       M4ERROR ((warning_status, 0,
-		"INTERNAL ERROR: Bad symbol type in call_macro ()"));
+		"INTERNAL ERROR: bad symbol type in call_macro ()"));
       abort ();
     }
 }
@@ -262,7 +294,7 @@ call_macro (symbol *sym, int argc, token_data **argv,
 | arguments, using collect_arguments (), and builds a table of pointers to |
 | the arguments.  The arguments themselves are stored on a local obstack.  |
 | Expand_macro () uses call_macro () to do the call of the macro.	   |
-| 									   |
+|									   |
 | Expand_macro () is potentially recursive, since it calls expand_argument |
 | (), which might call expand_token (), which might call expand_macro ().  |
 `-------------------------------------------------------------------------*/
@@ -270,36 +302,63 @@ call_macro (symbol *sym, int argc, token_data **argv,
 static void
 expand_macro (symbol *sym)
 {
-  struct obstack arguments;
-  struct obstack argptr;
+  struct obstack arguments;	/* Alternate obstack if argc_stack is busy.  */
+  unsigned argv_base;		/* Size of argv_stack on entry.  */
+  bool use_argc_stack = true;	/* Whether argc_stack is safe.  */
   token_data **argv;
   int argc;
   struct obstack *expansion;
   const char *expanded;
-  boolean traced;
+  bool traced;
   int my_call_id;
 
+  /* Report errors at the location where the open parenthesis (if any)
+     was found, but after expansion, restore global state back to the
+     location of the close parenthesis.  This is safe since we
+     guarantee that macro expansion does not alter the state of
+     current_file/current_line (dnl, include, and sinclude are special
+     cased in the input engine to ensure this fact).  */
+  const char *loc_open_file = current_file;
+  int loc_open_line = current_line;
+  const char *loc_close_file;
+  int loc_close_line;
+
+  SYMBOL_PENDING_EXPANSIONS (sym)++;
   expansion_level++;
-  if (expansion_level > nesting_limit)
+  if (nesting_limit > 0 && expansion_level > nesting_limit)
     M4ERROR ((EXIT_FAILURE, 0,
-	      "ERROR: Recursion limit of %d exceeded, use -L<N> to change it",
+	      "recursion limit of %d exceeded, use -L<N> to change it",
 	      nesting_limit));
 
   macro_call_id++;
   my_call_id = macro_call_id;
 
-  traced = (boolean) ((debug_level & DEBUG_TRACE_ALL) || SYMBOL_TRACED (sym));
+  traced = (debug_level & DEBUG_TRACE_ALL) || SYMBOL_TRACED (sym);
 
-  obstack_init (&argptr);
-  obstack_init (&arguments);
+  argv_base = obstack_object_size (&argv_stack);
+  if (obstack_object_size (&argc_stack) > 0)
+    {
+      /* We cannot use argc_stack if this is a nested invocation, and an
+	 outer invocation has an unfinished argument being
+	 collected.  */
+      obstack_init (&arguments);
+      use_argc_stack = false;
+    }
 
   if (traced && (debug_level & DEBUG_TRACE_CALL))
     trace_prepre (SYMBOL_NAME (sym), my_call_id);
 
-  collect_arguments (sym, &argptr, &arguments);
+  collect_arguments (sym, &argv_stack,
+		     use_argc_stack ? &argc_stack : &arguments);
 
-  argc = obstack_object_size (&argptr) / sizeof (token_data *);
-  argv = (token_data **) obstack_finish (&argptr);
+  argc = ((obstack_object_size (&argv_stack) - argv_base)
+	  / sizeof (token_data *));
+  argv = (token_data **) ((char *) obstack_base (&argv_stack) + argv_base);
+
+  loc_close_file = current_file;
+  loc_close_line = current_line;
+  current_file = loc_open_file;
+  current_line = loc_open_line;
 
   if (traced)
     trace_pre (SYMBOL_NAME (sym), my_call_id, argc, argv);
@@ -311,8 +370,18 @@ expand_macro (symbol *sym)
   if (traced)
     trace_post (SYMBOL_NAME (sym), my_call_id, argc, argv, expanded);
 
-  --expansion_level;
+  current_file = loc_close_file;
+  current_line = loc_close_line;
 
-  obstack_free (&arguments, NULL);
-  obstack_free (&argptr, NULL);
+  --expansion_level;
+  --SYMBOL_PENDING_EXPANSIONS (sym);
+
+  if (SYMBOL_DELETED (sym))
+    free_symbol (sym);
+
+  if (use_argc_stack)
+    obstack_free (&argc_stack, argv[0]);
+  else
+    obstack_free (&arguments, NULL);
+  obstack_blank (&argv_stack, -argc * sizeof (token_data *));
 }
